package org.numamo.services.rcm.data.adapter.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.Collections;

import static org.numamo.services.rcm.data.adapter.config.PropNames.SWAGGER_PROFILE_NAME;
import static springfox.documentation.builders.RequestHandlerSelectors.withClassAnnotation;


@Profile(SWAGGER_PROFILE_NAME)
@Configuration
@EnableSwagger2
public class SwaggerConfig {

    @Bean
    public Docket api() {

        return new Docket(DocumentationType.SWAGGER_2)
                .select()
                .apis(withClassAnnotation(RestController.class))
                .build()
                .apiInfo(apiInfo());
    }

    private ApiInfo apiInfo() {

        String title = "Car game data adapter used for processing of RCM (Researching Cognitive Maps)";
        String description = "The adapter reads user game results and executes custom calculations";
        String version = "0.1.0-RELEASE";
        String contactName = "Nazimov Aleksey Igorevich";
        String contactMail = "numamo@yandex.ru";
        String contactUrl = "https://www.numamo.org";
        String licenseTitle = "Apache-2.0";
        String licenseUrl = "http://www.apache.org/licenses/LICENSE-2.0.html";

        return new ApiInfo(
                title,
                description,
                version,
                null,
                new Contact(contactName,
                        contactUrl,
                        contactMail),
                licenseTitle,
                licenseUrl,
                Collections.emptyList()
        );
    }

}
