package org.numamo.services.rcm.data.adapter.controller.v2x.impl;

import org.numamo.rcm.car.game.protocols.v2x.dto.ResponseStatus;
import org.numamo.rcm.car.game.protocols.v2x.dto.basics.AbstractRequest;
import org.numamo.rcm.car.game.protocols.v2x.resume.GameResumeV2x;
import org.numamo.services.rcm.data.adapter.model.v2x.dto.ResponseV2x;
import org.numamo.services.rcm.data.adapter.model.v2x.dto.ResumeReportV2x;
import org.numamo.services.rcm.data.adapter.service.v2x.api.RequestControl;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ResumeProcessorFacade;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ResumeStorageService;
import org.slf4j.Logger;

import java.util.List;

import static java.util.Collections.emptyList;
import static org.numamo.rcm.car.game.protocols.v2x.dto.ResponseStatus.makeCustomError;
import static org.numamo.rcm.car.game.protocols.v2x.dto.ResponseStatus.success;
import static org.numamo.services.rcm.data.adapter.service.v2x.api.model.RequestParams.makeByRequest;


/**
 * The class represents the abstract controller algorithm
 *
 * @param <T> is a template of the controller request
 */
public abstract class AbstractProcessor<T extends AbstractRequest> {


    private final Logger logger = makeLogger();

    private final ResumeProcessorFacade processor;
    private final ResumeStorageService service;
    private final RequestControl requestControl;

    protected AbstractProcessor(ResumeProcessorFacade processor,
                                ResumeStorageService service,
                                RequestControl requestControl
    ) {
        this.processor = processor;
        this.service = service;
        this.requestControl = requestControl;
    }


    abstract Logger makeLogger();

    abstract List<GameResumeV2x> findResumes(T request, ResumeStorageService service);


    final ResponseV2x process(T request) {

        logger.info("Processing request with keywords: {}", request);

        final String requestId = requestControl
                .putCurrent(makeByRequest(request))
                .getCurrent()
                .getRequestId();

        final ResponseV2x responseV2x = new ResponseV2x();

        final List<GameResumeV2x> resumes = findResumes(request, service);
        if (resumes.isEmpty()) {
            responseV2x.setStatus(emptyResumeList(requestId));
            responseV2x.setReports(emptyList());

        } else {
            final List<ResumeReportV2x> reports = processor.process(resumes);
            responseV2x.setStatus(success(requestId));
            responseV2x.setReports(reports);
        }

        logger.trace("Final report: {}", responseV2x);
        return responseV2x;
    }


    private ResponseStatus emptyResumeList(String requestId) {
        return makeCustomError(1, "", "Requested resumes were not found", requestId);
    }


}
