package org.numamo.services.rcm.data.adapter.service.v2x.api.model;

import org.apache.commons.lang3.StringUtils;
import org.numamo.rcm.car.game.protocols.v2x.dto.basics.AbstractRequest;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.GetResumeRq;
import org.numamo.rcm.car.game.protocols.v2x.dto.resume.info.GetResumeInfoRq;

import java.util.UUID;

import static java.util.Objects.requireNonNull;


public final class RequestParams {

    private static final long NO_TOKEN = -1;
    private final long token;
    private final String requestId;

    private RequestParams(long token, String requestId) {
        this.token = token;
        this.requestId = requireNonNull(requestId);
    }

    public static <T extends AbstractRequest> RequestParams makeByRequest(T request) {

        if (GetResumeRq.class.isAssignableFrom(request.getClass())) {
            return new RequestParams(NO_TOKEN, processRequest(request));

        } else if (GetResumeInfoRq.class.isAssignableFrom(request.getClass())) {
            return new RequestParams(NO_TOKEN, processRequest(request));

        } else {
            throw new IllegalArgumentException("Unknown request class = " + request.getClass());
        }
    }

    private static <T extends AbstractRequest> String processRequest(T request) {
        final String requestId = request.getRequestId();
        return StringUtils.isBlank(requestId) ? UUID.randomUUID().toString() : requestId;
    }


    public long getToken() {
        return token;
    }

    public String getRequestId() {
        return requestId;
    }

    @Override
    public String toString() {
        return "RequestParams{" +
                "token=" + token +
                ", requestId='" + requestId + '\'' +
                '}';
    }

}
