package org.numamo.services.rcm.data.adapter.service.v2x.impl;

import org.apache.commons.lang3.StringUtils;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ConfigHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import static org.numamo.services.rcm.data.adapter.config.PropNames.*;


/**
 * This implementation is based on reading application.properties by the
 * standard Spring configs
 */
@Service
public final class ConfigHolderImpl implements ConfigHolder {


    //  Variables and constructors:------------------------------------------------------
    private final String mdcTokeTag;
    private final String mdcRequestIdTag;
    private final int requestControlStackSize;
    private final String keyWordV2xSearchUrl;
    private final String tokenV2xSearchUrl;

    @Autowired
    public ConfigHolderImpl(Environment environment) {

        /*
        If you want to change OPTIONALITY of some fields, PLEASE change config holder API for using the Optional.java
         */
        this.mdcTokeTag =
                read(environment, MDC_TOKEN_TAG, String.class, false, "");
        this.mdcRequestIdTag =
                read(environment, MDC_REQUEST_ID_TAG, String.class, false, "");
        this.requestControlStackSize =
                read(environment, REQUEST_CONTROL_STACK_SIZE, Integer.class, true, 10);

        this.keyWordV2xSearchUrl =
                read(environment, CGDS_KEY_WORD_SEARCH_URL_V2X, String.class, false, "");
        this.tokenV2xSearchUrl =
                read(environment, CGDS_TOKEN_SEARCH_URL_V2X, String.class, false, "");
    }


    // Public API:-----------------------------------------------------------------------
    @Override
    public int getMaxStackSizeOfRequestControl() {
        return requestControlStackSize;
    }

    @Override
    public String getMdcTokenTagName() {
        return mdcTokeTag;
    }

    @Override
    public String getMdcRequestTagName() {
        return mdcRequestIdTag;
    }

    @Override
    public String getSearchUrlWithKeyWordsV2x() {
        return keyWordV2xSearchUrl;
    }

    @Override
    public String getSearchUrlWithTokensV2x() {
        return tokenV2xSearchUrl;
    }


    // Internal methods:-----------------------------------------------------------------
    @SuppressWarnings("unchecked")
    private <T> T read(Environment environment,
                       String propName,
                       Class<T> clazz,
                       boolean isOptional,
                       T defaultValue
    ) {
        final String value = environment.getProperty(propName);
        if (StringUtils.isBlank(value)) {
            if (isOptional) {
                return defaultValue;
            } else {
                throw new IllegalStateException("Non optional value was not set." +
                        " Check your application properties for key=" + propName);
            }
        }

        if (String.class.isAssignableFrom(clazz)) {
            return (T) value;
        } else if (Integer.class.isAssignableFrom(clazz)) {
            return (T) Integer.valueOf(value);
        } else {
            throw new IllegalArgumentException("Requested class cannot be processed: " + clazz);
        }
    }


}
