package org.numamo.services.rcm.data.adapter.service.v2x.impl;

import org.numamo.services.rcm.data.adapter.service.v2x.api.ConfigHolder;
import org.numamo.services.rcm.data.adapter.service.v2x.api.RequestControl;
import org.numamo.services.rcm.data.adapter.service.v2x.api.model.RequestParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static java.util.Objects.isNull;
import static java.util.Objects.requireNonNull;


/**
 *
 */
@Service
public final class RequestControlImpl implements RequestControl {


    // Variables and constructors:-------------------------------------------------------
    private static final Logger LOGGER = LoggerFactory.getLogger(RequestControlImpl.class);
    private static final ThreadLocal<FixSizeStack> THREAD_LOCAL_HISTORY = new ThreadLocal<>();
    private static final ThreadLocal<RequestParams> CURRENT = new ThreadLocal<>();

    private final ConfigHolder config;

    @Autowired
    public RequestControlImpl(ConfigHolder config) {
        this.config = config;
    }


    // Public API:-----------------------------------------------------------------------
    @Override
    public RequestControl putCurrent(RequestParams request) {

        LOGGER.debug("Putting the new request control: {}", request);

        MDC.put(config.getMdcRequestTagName(), request.getRequestId());
        MDC.put(config.getMdcTokenTagName(), String.valueOf(request.getToken()));

        FixSizeStack stack = THREAD_LOCAL_HISTORY.get();
        if (isNull(stack)) {
            stack = new FixSizeStack(config.getMaxStackSizeOfRequestControl());
            THREAD_LOCAL_HISTORY.set(stack);
        }
        stack.add(request);

        CURRENT.set(request);

        if (LOGGER.isTraceEnabled()) {
            LOGGER.trace("Buffering of request control was performed: {}", request);
            LOGGER.trace("{} Thread statistics: {}", Thread.currentThread().getId(), stack);
        }
        return this;
    }

    @Override
    public RequestParams getCurrent() {
        return requireNonNull(CURRENT.get());
    }


    /**
     * Internal fixed stack
     */
    private static final class FixSizeStack {

        private final int maxSize;
        private final List<RequestParams> requestControls = new ArrayList<>();

        private FixSizeStack(int maxSize) {
            this.maxSize = maxSize;
        }

        private void add(RequestParams control) {
            if (requestControls.size() >= maxSize && maxSize > 0) {
                requestControls.remove(0);
            }
            requestControls.add(requireNonNull(control));
        }

        @Override
        public String toString() {
            return "FixSizeStack{" +
                    " size=" + requestControls.size() +
                    ", requestControls=" + requestControls +
                    '}';
        }
    }

}
