package org.numamo.services.rcm.data.adapter.service.v2x.impl;

import org.numamo.rcm.car.game.protocols.v2x.resume.GameResumeV2x;
import org.numamo.services.App;
import org.numamo.services.rcm.data.adapter.model.v2x.dto.ResumeReportV2x;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ResumeProcessor;
import org.numamo.services.rcm.data.adapter.service.v2x.api.ResumeProcessorFacade;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static java.util.Objects.requireNonNull;


@Service
public final class ResumeProcessorFacadeImpl implements ResumeProcessorFacade {


    // Variables and constructors:-------------------------------------------------------
    private static final Logger LOGGER = LoggerFactory.getLogger(ResumeProcessorFacadeImpl.class);

    private final ResumeProcessor resumeProcessor;

    @Autowired
    public ResumeProcessorFacadeImpl(List<ResumeProcessor> resumeProcessor) {
        this.resumeProcessor = selectProcessor(resumeProcessor);
    }


    // Public API:-----------------------------------------------------------------------
    @Override
    public List<ResumeReportV2x> process(List<GameResumeV2x> resumes) {

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("Processing resume list.size {}", resumes.size());
        }

        final List<ResumeReportV2x> reports = new ArrayList<>();
        for (final GameResumeV2x resume : resumes) {
            final Object result = resumeProcessor.process(resume);
            final ResumeReportV2x report = new ResumeReportV2x(resume.getToken(), result);

            if (LOGGER.isTraceEnabled()) {
                LOGGER.debug("Processing report.token={}", report.getToken());
            } else if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("Processing report {}", report);
            }

            reports.add(report);
        }

        return Collections.unmodifiableList(reports);
    }


    // Internal methods:-----------------------------------------------------------------
    private ResumeProcessor selectProcessor(List<ResumeProcessor> resumeProcessors) {

        if (resumeProcessors.isEmpty()) {
            throw new IllegalArgumentException("Now processors were defined!");

        } else if (resumeProcessors.size() == 1) {
            return requireNonNull(resumeProcessors.get(0), "The processor cannot be set as NULL");

        } else if (resumeProcessors.size() == 2) {
            final Optional<ResumeProcessor> p =
                    resumeProcessors.stream().filter((r) -> !(r instanceof App.ResumeProcessorExample)).findFirst();
            if (p.isPresent()) {
                return requireNonNull(p.get(), "The custom processor cannot be set as NULL");
            } else {
                throw new IllegalArgumentException("No custom processors were found");
            }

        } else {
            throw new IllegalArgumentException("There are more than 2 processors: n=" + resumeProcessors.size());
        }
    }

}
